﻿namespace AopDemo.Model.Tests {
    using System;
    using Entities;
    using log4net.Config;
    using Moq;
    using NUnit.Framework;

    [TestFixture]
    public class RepositoryTest {
        private readonly Mock<IDataAccessLayer> _mockDal = new Mock<IDataAccessLayer>();
        private Author _author;
        private Book _book;
        private IDataAccessLayer _dal;

        [TestFixtureSetUp]
        public void SetupTestFixture() {
            _author = new Author {
                                     Id = Guid.NewGuid(),
                                     FirstName = "Yan",
                                     LastName = "Cui",
                                     NickName = "theburningmonk"
                                 };
            _book = new Book {
                                 Id = Guid.NewGuid(),
                                 Author = _author,
                                 Title = "AOP is awesome",
                                 DatePublished = DateTime.Now,
                             };

            // set up the mock dal to always return the configured book and author
            _mockDal.Setup(dal => dal.Get<Book>(It.IsAny<Guid>())).Returns(_book);
            _mockDal.Setup(dal => dal.Get<Author>(It.IsAny<Guid>())).Returns(_author);
            _dal = _mockDal.Object;

            // Set up a simple configuration that logs on the console.
            BasicConfigurator.Configure();
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void RepositoryTestGetBookByIdWithDefaultGuid() {
            new Repository(_dal).GetBookById(default(Guid));
        }

        [Test]
        public void RepositoryTestGetBookByIdSuccess() {
            TestGetBookByIdSuccess(new Repository(_dal));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void AopRepositoryTestGetBookByIdWithDefaultGuid() {
            new AopRepository(_dal).GetBookById(default(Guid));
        }

        [Test]
        public void AopRepositoryTestGetBookByIdSuccess() {
            TestGetBookByIdSuccess(new AopRepository(_dal));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void RepositoryTestGetAuthoryIdWithDefaultGuid() {
            new Repository(_dal).GetAuthorById(default(Guid));
        }

        [Test]
        public void RepositoryTestGetAuthorByIdSuccess() {
            TestGetAuthorByIdSuccess(new Repository(_dal));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void AopRepositoryTestGetAuthorByIdWithDefaultGuid() {
            new AopRepository(_dal).GetBookById(default(Guid));
        }

        [Test]
        public void AopRepositoryTestGetAuthorByIdSuccess() {
            TestGetAuthorByIdSuccess(new AopRepository(_dal));
        }

        private void TestGetBookByIdSuccess(IRepository repository) {
            var book = repository.GetBookById(Guid.NewGuid());

            Assert.AreEqual(_book, book);
        }

        private void TestGetAuthorByIdSuccess(IRepository repository) {
            var author = repository.GetAuthorById(Guid.NewGuid());

            Assert.AreEqual(_author, author);
        }
    }
}
